#!/usr/bin/python
# coding: utf-8
#!/usr/bin/python
# coding: utf-8
# Luiz S Oliveira, Sept 2014
# It works with GPX files exported from Strava and it reads the following tags: <time> <gpxtpx:hr> cad, ele lat, lon
# Update: Sept 12 2014 (0.1.1) - histogram plot
# Update: Sept 14 2014 (0.1.2) - minidom
# Update: Sept 15 2014 (0.1.3) - fix hour to hh:mm:ss / ignore the first <time> tag
# Update: May 20 2015 (0.2) - uses HR data to compute the SS only if  the speed > 0
#			   - computes some time metrics (moving, pedaling, and stoped)
# Update: Jun 5 2016 (0.2.1) - python 3.0 compliance 


import matplotlib.pyplot as plt
import sys
import math
import numpy as np
from xml.dom.minidom import parse
from datetime import datetime

FMT = '%H:%M:%S'
SPD0   = 2  # below this threshold, the speed is set to zero
SPDMAX = 100 #above this threshold, it is considered a noise
limits = [94,113,133,140,174,191] # Define your zones here
coef   = [12.,24.,45.,100.,120.]
zones  = ['Endurance', 'Moderate ', 'Tempo    ', 'Threshold', 'Anaerobic' ]

##---------------------------------------------------------
## Class used to store all the GPS data
##---------------------------------------------------------

class GPS(object):

	def __init__(self):
		self.lat = [] 
		self.lon = [] 
		self.time =[] 
		self.hr = []
		self.cad = []
		self.ele = []
		self.speed = []

	def getLat(self, i):
		return float(self.lat[i])
		
	def getLon(self, i):
		return float(self.lon[i])

	def getTime(self, i):
		return self.time[i]

	def getHR(self, i):
		return self.hr[i]

	def getCad(self, i):
		return self.cad[i]

	def getEle(self, i):
		if (len(self.ele) == 0):
			return 0
		else:
			return self.ele[i]

	def setSpeed(self, v, seconds):
		for i in range(seconds):
			self.speed.append(v)	

	def getElevationInfo(self):
		return np.min(self.ele), max(self.ele)

	def getSpeedInfo(self):
		return np.mean(self.speed), np.std(self.speed), np.max(self.speed)






##-------------------------------------------------------------
## Histogram ploting
##-------------------------------------------------------------
def PlotHist(h,ss):

	if( ss== 0):
		print ('The SS is 0. Check the tags')
		return

	label = 'Peace of cake'
	if (ss > 100 and ss < 150):
		label = ' Tough'
	elif( ss>=150 and ss < 250 ):
		label  = ' Extreme'
	elif( ss >=250 ):
		label =  ' Epic'


	s = float(sum(h))
	data = [ ("Endurance", h[0]/s), ("Moderate", h[1]/s), ("Tempo",h[2]/s), ( "Threshold", h[3]/s), ("Anaerobic", h[4]/s)]
	N = len( data )
	x = np.arange(1, N+1)
	y = [ num for (s, num) in data ]
	labels = [ s for (s, num) in data ]
	width = 1
	bar1 = plt.bar( x, y, width, color="r" )
	title = 'Suffer Score: %d [%s]' % (ss, label)
	plt.title(title)
	plt.ylabel( '% of Time' )
	plt.xticks(x + width/2.0, labels )
	plt.show()


##-------------------------------------------------------------
## Loads GPX data
##-------------------------------------------------------------
def loaddata(dom, data):

	print ('Loading data...')

	coord = dom.getElementsByTagName('trkpt')
	for i in coord:
		data.lat.append(i.getAttribute('lat'))
		data.lon.append(i.getAttribute('lon'))


	data.hr.append(0)	
	hr = dom.getElementsByTagName('gpxtpx:hr')

	for i in hr:
		for t in i.childNodes:
			data.hr.append(int(t.data))

	data.cad.append(0)
	cad = dom.getElementsByTagName('gpxtpx:cad')
	for i in cad:
		for t in i.childNodes:
			data.cad.append(int(t.data))



	ele = dom.getElementsByTagName('ele')
	for i in ele:
		for t in i.childNodes:
			data.ele.append(float(t.data))

	
	time = dom.getElementsByTagName('time')
	for i in time:
		for t in i.childNodes:
			hh = t.data.split('T')
			data.time.append(hh[1][:8])

	data.time.pop(0) ## removes the first time info that does not belong to the track.



##-------------------------------------------------------------
## Computes the distance between two points
##-------------------------------------------------------------
def Distance(lat1, long1, lat2, long2):

	if lat1 == lat2 and long1 == long2:
		return 0.0 		
	#Convert latitude and longitude to spherical coordinates in radians.
	degrees_to_radians = math.pi/180.0
	 
	# phi = 90 - latitude
	phi1 = (90.0 - lat1)*degrees_to_radians
	phi2 = (90.0 - lat2)*degrees_to_radians
	 	
	# theta = longitude
	theta1 = long1*degrees_to_radians
	theta2 = long2*degrees_to_radians
	 
	#print lat1, long1, lat2, long2
	#print -- phi1, phi2, theta1, theta2 
 
	# Compute spherical distance from spherical coordinates.
	cos = (math.sin(phi1)*math.sin(phi2)*math.cos(theta1 - theta2) + math.cos(phi1)*math.cos(phi2))
	#print lat1, long1, lat2, long2, cos 
	arc = math.acos( cos )
 
	# Multiply arc by the radius of the earth in your favorite set of units to get length.
	return arc * 6373

##-------------------------------------------------------------
## Computes Elevation stuff
##-------------------------------------------------------------
def ComputeElevation(vetEle):

	up = 0.0
	down = 0.0
	tUp = 0 
	tDown = 0
	tFlat = 0.0
	#print '0', vetEle[0]
	for i in range(1, len(vetEle)):
		dif = vetEle[i] - vetEle[i-1]
		if dif > 0:
			up = up + dif
			tUp = tUp + 1
		elif dif < 0:
			down = down + (dif * -1.0)
			tDown = tDown + 1
		else:
			tFlat = tFlat+1 ;
		#print i, vetEle[i], dif, up, down

	
	#print up, down

	#m, s = divmod(tUp, 60)
	#h, m = divmod(m, 60)
	#print "Up	     : %d:%02d:%02d" % (h, m, s)

	#m, s = divmod(tDown, 60)
	#h, m = divmod(m, 60)
	#print "Down	   : %d:%02d:%02d" % (h, m, s)

	#m, s = divmod(tFlat, 60)
	#h, m = divmod(m, 60)
	#print "Flat	   : %d:%02d:%02d" % (h, m, s)

	#m, s = divmod(tFlat+tUp+tDown, 60)
	#h, m = divmod(m, 60)
	#print "Total	  : %d:%02d:%02d" % (h, m, s)
##-------------------------------------------------------------
## Main program
##-------------------------------------------------------------
def main(filein):

	data = GPS()
	tSleep = 0
	tMoving = 0
	tPedaling = 0
	tTotal = 0
	vetCad = []
	vetHR = []
	vetEle = []
	zonesHR = [0,0,0,0,0]
	print ('Parsing GPX...')
	dom = parse(filein)
	loaddata(dom, data )

	size = len(data.lat)
	for i in range(1,size):

		## computes the distance between two points
		dist = Distance(data.getLat(i), data.getLon(i), data.getLat(i-1), data.getLon(i-1))
		diff = datetime.strptime(data.getTime(i), FMT) - datetime.strptime(data.getTime(i-1), FMT)
		tTotal = tTotal + diff.seconds

		## computes the distance
		speed = 0
		if diff.seconds > 0:
			speed = ((dist / diff.seconds) * 3600.) 
			if speed < SPD0:
				speed = 0
			if speed > SPDMAX:
				speed = 0
	
		if speed == 0:
			#print 'ZERO'
			tSleep = tSleep + diff.seconds
		else:
			tMoving = tMoving + diff.seconds
			data.setSpeed(speed, diff.seconds)

		## Cadence
		if data.getCad(i) > 0:
			tPedaling = tPedaling + diff.seconds
			for x in range( diff.seconds):
				vetCad.append(data.getCad(i))

		## Heart rate and Elevation
		if speed > 0:
			for x in range(diff.seconds):
				vetHR.append(data.getHR(i))
				vetEle.append(data.getEle(i))

			for j in range(5):
				if( data.getHR(i) < limits[j+1]):
					zonesHR[j] = zonesHR[j] + diff.seconds
					break 

	## Elevation
	ComputeElevation( vetEle )		
			
 	
	m, s = divmod(tTotal, 60)
	h, m = divmod(m, 60)
	print ('Summary:')
	print ('----------------------------------------------------------------------')
	print ("Total Time     : %d:%02d:%02d" % (h, m, s))

	m, s = divmod(tMoving, 60)
	h, m = divmod(m, 60)
	print (". Moving       : %d:%02d:%02d - %f" % (h, m, s, (float(tMoving) /float(tTotal))*100))

	m, s = divmod(tPedaling, 60)
	h, m = divmod(m, 60)
	print (". Pedaling     : %d:%02d:%02d" % (h, m, s))

	m, s = divmod(tSleep, 60)
	h, m = divmod(m, 60)
	print (". Sleeping     : %d:%02d:%02d" % (h, m, s))


	print ('Min Elev       : %f, Max Elev: %f' %  data.getElevationInfo())
	print ('Avg Speed      : %f, Std: %f, Max Speed: %f' %  data.getSpeedInfo())
	print ('Avg Cad	:', np.mean(vetCad))
	print ('Max Cad	:', np.max(vetCad))
	
	print ('Avg HR	 :', np.mean(vetHR))
	print ('Min HR	 :', np.min(vetHR))
	print ('Max HR	 :', np.max(vetHR))

	ss = 0.
	for i in range (5):
		ss = ss+ (coef[i] * (zonesHR[i]/3600.))
	print('Suffer Score   :', ss)

	print('----------------------------------------------------------------------')
	PlotHist(zonesHR, ss)

	
if __name__ == "__main__":
	if len(sys.argv) != 2:
		sys.exit("Usage: domss.py <gpx file>")
	main(sys.argv[1])


